/*****************************************************************************
MODULE: redblack.h
PROJECT: redblack
PROGRAMMER: Jeff Nielsen
DATE: February 1996

DESCRIPTION: This header declares the classes for a red-black tree--a variant
             of the Binary Search Tree.

	     The red-black tree can be visualized by linking with the Swan
	     Annotation Interface Library (SAIL), and by defining the macro
	     SWAN.

*****************************************************************************/
#ifndef _REDBLACK_H
#define _REDBLACK_H

#include <stdio.h>
#include "stdjeff.h"

#define MIN 1		      // Minimum & maximum values allowed in tree
#define MAX 999
#define ERRSTREAM "ERROR: Value must be between " << MIN << "-" << MAX << "."

enum COLORTYPE {RED, BLCK};   // The definition BLACK is taken by Swan
typedef int ELEM;

// rb_node class definition --------------------------------------------------
class rb_node {
public:
   ELEM key;                     // The node's value
   COLORTYPE color;              // The node's color--RED or BLCK
   rb_node* left;                // Pointer to left child
   rb_node* right;               // Pointer to  right child
   rb_node* p;                   // Pointer to parent node

   static rb_node* freelist;     // Pointer to head of freelist
   void* operator new(size_t);   // Overload new & delete to
   void operator delete(void*);  //    implement freelist

   rb_node()   // Default constructor--used only to initialize NULLNODE
      { color = BLCK; }
   rb_node(ELEM, COLORTYPE =RED);   // Constructor used to specify key
   ~rb_node() { }
};

// redblack_tree class definition --------------------------------------------
class redblack_tree {
private:
   rb_node* root;
   bool newline;                    // Variable for printhelp

   void clearhelp(rb_node*);        // Helper functions for public methods
   void inserthelp(rb_node*&, const ELEM, rb_node*);
   void removehelp(rb_node*, const ELEM);
// rb_node* findhelp(rb_node*&, const ELEM, int, int);
   void printhelp(const rb_node*, const int);
   void left_rotate(rb_node*);
   void right_rotate(rb_node*);
   void fix_tree(rb_node*);
   void delete_fixup(rb_node* x);

public:
   redblack_tree();		                // Constructor
   ~redblack_tree() { clearhelp(root); }        // Destructor
   void clear();                         // Clear the tree
   void insert(const ELEM& val);                // Insert value in tree
   void remove(const ELEM& val);                // Remove value from tree
   rb_node* findmin(rb_node*);
// rb_node* find(const ELEM& val) {  // Search for value in tree
//    if (val >= MIN && val <= MAX) return findhelp(root, val);
//    else { cout << ERRSTREAM; return NULL; }
// }
   rb_node* get_root() { return root; }
   bool is_empty() const;
   void print() {
      if (is_empty()) cout << "The tree is empty.";
      else { newline = TRUE; printhelp(root, 0); }
   }
};

#endif
